package drawingboard;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.smardec.mousegestures.MouseGestures;
import com.smardec.mousegestures.MouseGesturesListener;

public class DrawingBoard extends JFrame{
	
	protected static URLClassLoader urlLoader = (URLClassLoader)ClassLoader.getSystemClassLoader();
	protected static URL load(String file) {return urlLoader.findResource(file);}
	
	private static final long serialVersionUID = 1L;
	private DrawingPane contentPane=null;
	
	public DrawingBoard(){
		super();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		contentPane=new DrawingPane();
		add(contentPane);
		pack();
	}
	
	
	class DrawingPane extends JPanel implements MouseGesturesListener,MouseListener,MouseMotionListener{
		
		private static final long serialVersionUID = 1L;
		private MouseGestures mouseGestures = null;
		private BufferedImage painting=null;
		private BufferedImage toile=null;
		private boolean mousePressed=false;
		//zone: pt Origine/pt le + eloigné/dernier pt enregistré/pt courant
		private Point zone[]={new Point(),new Point(),new Point(),new Point()};
		//variable tampon
		private double area;
		
		
		public DrawingPane(){
			
			try {
				toile=ImageIO.read(load("images/toile.png"));
			} catch (IOException e){
				e.printStackTrace();
			}
			setPreferredSize(new Dimension(toile.getWidth(),toile.getHeight()));
			painting=new BufferedImage(toile.getWidth(),toile.getHeight(),BufferedImage.TYPE_INT_RGB);;
			Graphics2D g2d = painting.createGraphics();
			g2d.setColor(Color.WHITE);
			g2d.fillRect(0,0,toile.getWidth(),toile.getHeight());
			addMouseListener(this);
			addMouseMotionListener(this);
			mouseGestures = new MouseGestures();
			mouseGestures.setMouseButton(MouseEvent.BUTTON1_MASK);
			mouseGestures.addMouseGesturesListener(this);
			mouseGestures.start();
		}

		public void gestureMovementRecognized(String arg0){}

		public void processGesture(String arg0) {
			Shape shape=null;
			System.out.println(arg0);
			if(arg0.equals("DRUL")) {
				shape=new Rectangle(new Dimension(zone[1].x-zone[0].x,zone[1].y-zone[0].y));
				((Rectangle)shape).setLocation(new Point(5,5));
				repaint();
			} else if(arg0.equals("LDRUL")){
				
				shape=new Ellipse2D.Double(5,5,zone[1].x-zone[0].x,zone[1].y-zone[0].y);
				System.out.println("Cercle");
			}
			if(shape!=null){
				float[] filter={ 	0,.2f,0,
									.2f,.2f,.2f,
									0,.2f,0	};
				BufferedImageOp blur=new ConvolveOp(new Kernel(3, 3, filter));
				BufferedImage tmp=new BufferedImage(shape.getBounds().width+10,shape.getBounds().height+10,BufferedImage.TYPE_INT_ARGB);
				Graphics2D g2d=tmp.createGraphics();
				g2d.drawImage(painting,
						0,0,tmp.getWidth(),tmp.getHeight(),
						zone[0].x-5,zone[0].y-5,zone[0].x-5+tmp.getWidth(),zone[0].y-5+tmp.getHeight(),
						null);
				g2d.setColor(new Color((int)(255*Math.random()),(int)(255*Math.random()),(int)(255*Math.random())));
				g2d.fill(shape);
				
				tmp=blur.filter(tmp,null);
				tmp=blur.filter(tmp,null);
				tmp=blur.filter(tmp,null);
				
				g2d=painting.createGraphics();
				AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,.8f);
				g2d.setComposite(ac);
				g2d.drawImage(tmp,
						zone[0].x-3,zone[0].y-3,zone[0].x-3+tmp.getWidth()-4,zone[0].y-3+tmp.getHeight()-4,
						2,2,tmp.getWidth()-2,tmp.getHeight()-2,
						null);
			}
			mousePressed=false;
			repaint();
		}
		
		protected void paintComponent(Graphics g){
			Graphics2D g2d=(Graphics2D)g;
			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
			if(!mousePressed){
				g2d.drawImage(painting, null, 0,0);
				g2d.drawImage(toile, null, 0,0);
			}else{
				g2d.setColor(Color.BLUE);
				g2d.setStroke(new BasicStroke(2.0f));
				g2d.drawLine(zone[2].x,zone[2].y,zone[3].x,zone[3].y);
				zone[2]=zone[3];
			}
		}

		public void mouseClicked(MouseEvent arg0){}
		public void mouseEntered(MouseEvent arg0){}
		public void mouseExited(MouseEvent arg0){}
		public void mousePressed(MouseEvent arg0){
			mousePressed=true;
			zone[2].x=zone[3].x=zone[1].x=zone[0].x=arg0.getX();
			zone[2].y=zone[3].y=zone[1].y=zone[0].y=arg0.getY();
			area=0;
		}
		public void mouseReleased(MouseEvent arg0){}
		public void mouseDragged(MouseEvent arg0){
			if(mousePressed){
				zone[3]=arg0.getPoint();
				double tmpArea=Math.sqrt(Math.pow((arg0.getX()-zone[0].x)*(arg0.getY()-zone[0].y),2));
				if(area<tmpArea){
					zone[1]=arg0.getPoint();
					area=tmpArea;
				}
				repaint();
			}
		}
		public void mouseMoved(MouseEvent arg0){
			
		}
		
	}
	


}
