package drawingBoard;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.smardec.mousegestures.MouseGestures;
import com.smardec.mousegestures.MouseGesturesListener;


public class DrawingBoard extends JFrame{
	private static final long serialVersionUID = 1L;
	private static final String ERASEGESTURE=""; //16)
	private static final String CIRCLEGESTURE=""; //16)
	protected static URLClassLoader urlLoader = (URLClassLoader)ClassLoader.getSystemClassLoader();
	protected static URL load(String file) {return urlLoader.findResource(file);}
	
	
	private DrawingPane contentPane=null;
	
	public DrawingBoard(){
		super();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		//22)
		contentPane=new DrawingPane();
		add(contentPane);
		pack();
	}
	
	
	class DrawingPane extends JPanel implements MouseGesturesListener,MouseListener,MouseMotionListener{
		private static final long serialVersionUID = 1L;
		
		//3 états pour la machine à état
		private static final int DEFAULT = 0;
		private static final int CIRCLE = 2;
		private static final int GESTURE = 4;
		//état courant
		private int mode;
		
		private BufferedImage painting=null;
		private BufferedImage toile=null;
		
		//endroit ou l'on clique au debut de l'action
		private Point origin;
		//dernier point enregistré et pointeur courant. Utile pour dessiner le geste en cours (trait bleu)
		private Point lastPoint, currentPoint;
		//forme en train d'etre dessinée
		private Shape currentShape;
		
		private Vector<Shape> shapes;
		
		public DrawingPane(){
			
			try {
				toile=ImageIO.read(load("images/toile.png"));
			} catch (IOException e){
				e.printStackTrace();
			}
			
			shapes=new Vector<Shape>();
			mode=DEFAULT;
			
			//initialisation de la taille de la fenetre à la taille de la toile
			setPreferredSize(new Dimension(toile.getWidth(),toile.getHeight()));
			//dessin du fond du tableau dans la bufferedImage painting
			paintCanvas();
			
			//ajout de l'écoutes des évènements
			addMouseListener(this);
			addMouseMotionListener(this);
			
			//ajout de l'écoute d'un nouvel evenement Gesture
			MouseGestures mouseGestures = new MouseGestures();
			mouseGestures.setMouseButton(MouseEvent.BUTTON1_MASK);
			
			mouseGestures.addMouseGesturesListener(this);
			mouseGestures.start();
			
		}

		public void gestureMovementRecognized(String arg0){//rien à faire içi
		}
		
		protected void paintCanvas(){		
			painting=new BufferedImage(toile.getWidth(),toile.getHeight(),BufferedImage.TYPE_INT_RGB);
			Graphics2D canvas=painting.createGraphics();
			canvas.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
			canvas.setColor(Color.WHITE);
			canvas.fillRect(0,0,toile.getWidth(),toile.getHeight());
			
			for(Shape shape:shapes){
				canvas.setColor(Color.BLUE);
				//21)
				canvas.fill(shape);
			}

		}
		
		protected void paintComponent(Graphics g){
			Graphics2D g2d=(Graphics2D)g;
			
			if(mode==GESTURE){
				//dessin du geste en cours
				//20
				//20
				//20
			}else if(mode==DEFAULT || mode==CIRCLE){
				//dessin du fond de la peinture
				g2d.drawImage(painting, null, 0,0);
				g2d.drawImage(toile, null, 0,0);
				g2d.setColor(Color.RED);
				if(mode==CIRCLE){
					//dessin du cercle de mesure en rouge
					//17)
					//17)
					
					//18)
				}
			}
		}

		public void mouseClicked(MouseEvent arg0){
			if(mode==CIRCLE){
				//18)
				mode=DEFAULT;
				paintCanvas();
				repaint();
			}
		}
		public void mouseEntered(MouseEvent arg0){}
		public void mouseExited(MouseEvent arg0){}
		public void mousePressed(MouseEvent arg0){
			if(mode==DEFAULT){
				mode=GESTURE;
				origin=arg0.getPoint();
				lastPoint=currentPoint=origin;
				repaint();
			}
		}
		public void mouseReleased(MouseEvent arg0){}
		public void mouseDragged(MouseEvent arg0){
			if(mode==GESTURE){
				lastPoint=currentPoint;
				currentPoint=arg0.getPoint();
				repaint();
			}
		}
		public void mouseMoved(MouseEvent arg0){
			if(mode==CIRCLE){
				currentPoint=arg0.getPoint();
				repaint();
			}
		}
		
		public void processGesture(String arg0) {
				if(arg0.equals(CIRCLEGESTURE)) {
					mode=CIRCLE;
				} else if(arg0.contains(ERASEGESTURE)){
					//19)
					//19)
					//19)
					//19)
					//19)
					mode=DEFAULT;
					//19)
					//19)
				} else {
					mode=DEFAULT;
					repaint();
				}
		}
	}
	
	class Circle extends Ellipse2D.Double {
		private static final long serialVersionUID = 1L;

		public Circle(double centerX, double centerY, double radius) {
			super(centerX - radius, centerY - radius, 2.0*radius, 2.0*radius);
		}
	}
}
